import { execSync } from 'child_process';
import { readFileSync, writeFileSync } from 'fs';
import path from 'path';

export default function publish(name: string, dirname: string, src = 'out-tsc', beta = false) {
  const version = process.argv[2];
  const semverRegex = /^([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?(?:\+[0-9A-Za-z-]+)?$/;

  // Version arg required
  if (!version) process.exit();

  // Version arg format check
  if (!version.match(semverRegex)) {
    console.error('Please enter a semantic version (e.g. 1.0.0).');
    process.exit();
  }

  // Version property update
  const srcPath = path.resolve(dirname, `../${src}/`, `${name}.js`);
  let srcString = readFileSync(srcPath, 'utf8');
  const replaceRegex = /get version[\s\S]*?'([\d\.\-betalph]*?)';/;
  const matches = replaceRegex.exec(srcString);
  if (matches) {
    srcString = srcString.replace(matches[0], matches[0].replace(matches[1], version));
    writeFileSync(srcPath, srcString);

    // Commit version property update
    execSync(`git commit -a -m "v${version}"`);
  } else {
    console.error('Version property not found.');
    process.exit();
  }

  // NPM version update
  execSync(`npm version ${version}`);

  // Publish to npm
  execSync(`npm publish ${beta ? '--tag beta' : ''} --registry https://registry.npmjs.org/`);
  execSync(`npm publish ${beta ? '--tag beta' : ''} --registry https://npm.pkg.github.com/vaadin-component-factory`);

  // Clean up git logs
  execSync('git reset HEAD~2');
  execSync(`git commit -a -m "v${version}"`);

  // Push to upstream
  execSync('git push');
  execSync(`git push origin v${version}`);
}
